#include "Symbol.h"
#include "Graph.h"
#include <string>
#include <iostream>
#include <map>
#include <vector>
#include <fstream>

using namespace std;


void Tokenize(const string& str, vector<string>& tokens, const string& delimiters = "/")
{
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (string::npos != pos || string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}


SymbolGraph::SymbolGraph()
{
	vector<string> tokens;
	string line;
	map<string,int>::iterator j;


	//Build Symbol Graph
	//Load movies.txt
	ifstream FileIn("movies.txt");

	if(!FileIn)
	{
		cout << "Can't Open movies file !!" << endl;
		exit(1);
	}

	cout << "Please wait while graph builds and BFS runs" << endl;

	while(getline(FileIn,line))
	{
		Tokenize(line, tokens); 
		for(int i =0; i < tokens.size(); i++)
		{
			if(st.find(tokens[i]) == st.end())
			{
				st.insert(pair<string,int>(tokens[i],(int)st.size()));
			}
		}
		tokens.clear();
	}
	FileIn.close();


	//Build Keys
	keys.resize(st.size());
	for(j = st.begin(); j != st.end(); j++)
	{
		keys[j->second] = j->first;
	}



	//Build Graph
	ifstream File2In("movies.txt");

	if(!File2In)
	{
		cout << "Can't Open movies file !!" << endl;
		exit(1);
	}


	G = new Graph((int)st.size());

	while(getline(File2In,line))
	{
		Tokenize(line, tokens); 
		
		j = st.find(tokens[0]);

		int v = j->second;

		for(int i =1; i < tokens.size(); i++)
		{
			j = st.find(tokens[i]);

			int w = j->second;
			G->addEdge(v,w);
		}
		tokens.clear();
	}
	File2In.close();

}

SymbolGraph::~SymbolGraph()
{

}

bool SymbolGraph::contains(string s)
{
	if(st.find(s) != st.end())
	{
		return true;
	}
	return false;
}

int SymbolGraph::index(string s)
{
	map<string,int>::iterator i;
	if(st.find(s) != st.end())
	{
		i = st.find(s);
		return i->second;
	}
	return -1;

}

string SymbolGraph::name(int v)
{
	return keys[v];
}

Graph SymbolGraph::getGraph()
{
	return *G;
}